﻿using PersonnelDepartmentContracts.BindingModels;
using PersonnelDepartmentContracts.BusinessLogicContracts;
using PersonnelDepartmentContracts.SearchModels;
using PersonnelDepartmentContracts.StoragesContracts;
using PersonnelDepartmentContracts.ViewModels;
using PersonnelTypeContracts.StoragesContracts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PersonnelDepartmentBusinessLogic.BusinessLogics
{
	public class TypeLogic : ITypeLogic
	{
		private readonly ITypeStorage _typeStorage;

		public TypeLogic(ITypeStorage typeStorage)
		{
			_typeStorage = typeStorage ?? throw new ArgumentNullException(nameof(typeStorage));
		}

		public bool Create(TypeBindingModel model)
		{
			CheckModel(model);
			if (_typeStorage.Insert(model) == null)
			{
				return false;
			}
			return true;
		}

		public bool Delete(TypeBindingModel model)
		{
			CheckModel(model);
			if (_typeStorage.Delete(model) == null)
			{
				return false;
			}
			return true;
		}

		public TypeViewModel? ReadElement(TypeSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			var element = _typeStorage.GetElement(model);
			if (element == null)
			{
				return null;
			}
			return element;
		}

		public List<TypeViewModel>? ReadList(TypeSearchModel? model)
		{
			var list = model == null ? _typeStorage.GetFullList() : _typeStorage.GetFilteredList(model);
			if (list == null)
			{
				return null;
			}
			return list;
		}

		public bool Update(TypeBindingModel model)
		{
			CheckModel(model);
			if (_typeStorage.Update(model) == null)
			{
				return false;
			}
			return true;
		}

		private void CheckModel(TypeBindingModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (string.IsNullOrEmpty(model.Name))
			{
				throw new ArgumentException("Отсутвует имя сотрудника",
					nameof(model.Name));
			}
			if (_typeStorage.GetElement(new TypeSearchModel
			{
				Name = model.Name
			}) != null)
			{
				throw new InvalidOperationException("Сотрудник с такими атрибутами уже есть");
			}
		}
	}
}
