﻿using PersonnelDepartmentContracts.BindingModels;
using PersonnelDepartmentContracts.SearchModels;
using PersonnelDepartmentContracts.ViewModels;
using PersonnelDepartmentDatabaseImplement.Models;
using PersonnelTypeContracts.StoragesContracts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Type = PersonnelDepartmentDatabaseImplement.Models.Type;

namespace PersonnelDepartmentDatabaseImplement.Implements
{
	public class TypeStorage : ITypeStorage
	{
		public TypeViewModel? Delete(TypeBindingModel model)
		{
			using var context = new PersonnelDepartmentDatabase();
			var element = context.Types
				.FirstOrDefault(x => x.Id == model.Id);
			if (element == null)
			{
				return null;
			}
			context.Types.Remove(element);
			return element.GetViewModel;
		}

		public TypeViewModel? GetElement(TypeSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return null;
			}
			using var context = new PersonnelDepartmentDatabase();
			return context.Types
				.FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id) ||
				(!string.IsNullOrEmpty(model.Name) && x.Name == model.Name))?.GetViewModel;
		}

		public List<TypeViewModel> GetFilteredList(TypeSearchModel model)
		{
			using var context = new PersonnelDepartmentDatabase();
			return context.Types
				.Where(x => (model.Id.HasValue && x.Id == model.Id) ||
				(!string.IsNullOrEmpty(model.Name) && x.Name == model.Name))
				.Select(x => x.GetViewModel).ToList();
		}

		public List<TypeViewModel> GetFullList()
		{
			using var context = new PersonnelDepartmentDatabase();
			return context.Types
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public TypeViewModel? Insert(TypeBindingModel model)
		{
			var newElement = Type.Create(model);
			if (newElement == null)
			{
				return null;
			}
			using var context = new PersonnelDepartmentDatabase();
			context.Types.Add(newElement);
			context.SaveChanges();
			return newElement.GetViewModel;
		}

		public TypeViewModel? Update(TypeBindingModel model)
		{
			using var context = new PersonnelDepartmentDatabase();
			var element = context.Types
				.FirstOrDefault(x => x.Id == model.Id);
			if (element == null)
			{
				return null;
			}
			element.Update(model);
			context.SaveChanges();
			return element.GetViewModel;
		}
	}
}
